import sgqlc.types


pdp_schema = sgqlc.types.Schema()



########################################################################
# Scalars and Enumerations
########################################################################
Boolean = sgqlc.types.Boolean

Int = sgqlc.types.Int

String = sgqlc.types.String


########################################################################
# Input Objects
########################################################################

########################################################################
# Output Objects and Interfaces
########################################################################
class FaroItem(sgqlc.types.Type):
    __schema__ = pdp_schema
    __field_names__ = ('id', 'program_id', 'item_nr', 'media_urns', 'play_links', 'descriptor_paths', 'program', 'title', 'producer', 'description', 'descriptor', 'sport_doc', 'persons')
    id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='id')
    program_id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='programId')
    item_nr = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='itemNr')
    media_urns = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))), graphql_name='mediaUrns')
    play_links = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))), graphql_name='playLinks')
    descriptor_paths = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))), graphql_name='descriptorPaths')
    program = sgqlc.types.Field('FaroProgram', graphql_name='program')
    title = sgqlc.types.Field(String, graphql_name='title')
    producer = sgqlc.types.Field(String, graphql_name='producer')
    description = sgqlc.types.Field(String, graphql_name='description')
    descriptor = sgqlc.types.Field(String, graphql_name='descriptor')
    sport_doc = sgqlc.types.Field(sgqlc.types.non_null('FaroSportDoc'), graphql_name='sportDoc')
    persons = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('Person'))), graphql_name='persons')


class FaroItemPage(sgqlc.types.Type):
    __schema__ = pdp_schema
    __field_names__ = ('edges', 'cursor')
    edges = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(FaroItem))), graphql_name='edges')
    cursor = sgqlc.types.Field(String, graphql_name='cursor')


class FaroModerator(sgqlc.types.Type):
    __schema__ = pdp_schema
    __field_names__ = ('first_name', 'last_name')
    first_name = sgqlc.types.Field(String, graphql_name='firstName')
    last_name = sgqlc.types.Field(String, graphql_name='lastName')


class FaroProgram(sgqlc.types.Type):
    __schema__ = pdp_schema
    __field_names__ = ('id', 'business_unit', 'media_type', 'episode_ids', 'date', 'show_name', 'title', 'series_nr', 'moderators')
    id = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='id')
    business_unit = sgqlc.types.Field(String, graphql_name='businessUnit')
    media_type = sgqlc.types.Field(String, graphql_name='mediaType')
    episode_ids = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))), graphql_name='episodeIds')
    date = sgqlc.types.Field(String, graphql_name='date')
    show_name = sgqlc.types.Field(String, graphql_name='showName')
    title = sgqlc.types.Field(String, graphql_name='title')
    series_nr = sgqlc.types.Field(Int, graphql_name='seriesNr')
    moderators = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(FaroModerator))), graphql_name='moderators')


class FaroSportDoc(sgqlc.types.Type):
    __schema__ = pdp_schema
    __field_names__ = ('season', 'sport_event_paths', 'sport_paths')
    season = sgqlc.types.Field(String, graphql_name='season')
    sport_event_paths = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))), graphql_name='sportEventPaths')
    sport_paths = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(String))), graphql_name='sportPaths')


class Person(sgqlc.types.Type):
    __schema__ = pdp_schema
    __field_names__ = ('first_name', 'last_name', 'alias_first_name', 'alias_last_name', 'role')
    first_name = sgqlc.types.Field(String, graphql_name='firstName')
    last_name = sgqlc.types.Field(String, graphql_name='lastName')
    alias_first_name = sgqlc.types.Field(String, graphql_name='aliasFirstName')
    alias_last_name = sgqlc.types.Field(String, graphql_name='aliasLastName')
    role = sgqlc.types.Field(String, graphql_name='role')


class Query(sgqlc.types.Type):
    __schema__ = pdp_schema
    __field_names__ = ('faro_items',)
    faro_items = sgqlc.types.Field(sgqlc.types.non_null(FaroItemPage), graphql_name='faroItems', args=sgqlc.types.ArgDict((
        ('first', sgqlc.types.Arg(sgqlc.types.non_null(Int), graphql_name='first', default=None)),
        ('after', sgqlc.types.Arg(String, graphql_name='after', default=None)),
))
    )



########################################################################
# Unions
########################################################################

########################################################################
# Schema Entry Points
########################################################################
pdp_schema.query_type = Query
pdp_schema.mutation_type = None
pdp_schema.subscription_type = None




# -*- coding: utf-8 -*-

"""Top-level package for SRGSSR Publication Data GraphQL client"""

__author__ = """PDP"""
__email__ = 'pdp@srgssr.ch'
__version__ = '0.4.1'

from srgssr_publication_data_api import pdp_schema
from sgqlc.endpoint.http import HTTPEndpoint
from base64 import b64encode
from sgqlc.operation import Operation

class PublicationDataApi:
    """Class for using SRG/SSR Publication Data API"""

    def __init__(self, url, username, password):
        if (url is None):
            raise ValueError("URL must be set for the API to work")
        self.url = url
        self.schema = pdp_schema
        self.endpoint = self.get_http_endpoint(url, username, password) # TODO check HTTPEndpoint safety here...

    def run_query(self, query, variables=None):
        """Queries GraphQL API with input query and returns the transformed result"""
        result = self.call_api(query, variables)

        # verify if errors were returned
        if "errors" in result:
            combined_error = ', '.join([ error['message'] for error in result['errors'] ])
            raise PublicationDataApiException(f'API returned with errors: {combined_error}')

        # Transform the result back into the schema
        return query + result

    def query_op(self, name=None, **args):
        """Create the sgqlc operation with the schema from the Publication Data API.
        see :func:`sgqlc.operation.Operation`
        """
        return Operation(self.schema.Query, name, **args)


    def call_api(self, query, variables):
        """Calls GraphQL endpoint with sgqlc http endpoint"""
        return self.endpoint(query, variables)


    def get_http_endpoint(self, url, username, password):
        """Gets the GraphQL Endpoint object to query from parameters"""
        if (username and password):
            headers = self.create_basic_auth_headers(username, password)
        else:
            headers = None
        endpoint = HTTPEndpoint(url, headers)
        return endpoint


    def create_basic_auth_headers(_, username, password):
        """Assembles Basic HTTP Authentication Header from parameters"""
        joined_credentials = b':'.join((username.encode('latin1'),
                                        password.encode('latin1')))
        auth = 'Basic ' + b64encode(joined_credentials).decode("ascii")
        headers = {'Authorization': auth}
        return headers

class PublicationDataApiException(Exception):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `srgssr_publication_data_api` package."""

from sgqlc.operation import Operation
import pytest

from srgssr_publication_data_api import PublicationDataApi, PublicationDataApiException
from unittest.mock import patch

EXAMPLE_ENDPOINT = "https://graphql-api.example.com/graphql"
EXAMPLE_RESPONSE_OK = {
 "data": {
  "assets": [
   {
    "assetId": "test",
    "title": "test",
    "orientation": None,
    "hasContributor": [
     {
      "givenName": "test",
      "familyName": "test"
     }
    ]
   }
  ]
 }
}
EXAMPLE_RESPONSE_ERROR = {
 "data": None,
 "errors": [
  {
   "message": "Malformed Query!"
  }
 ]
}

def test_assemble_authorization_headers():
    """Test assembly of basic HTTP authentication headers"""
    client = PublicationDataApi("no-url", "dummy", "password")
    headers = client.endpoint.base_headers
    assert 'Authorization' in headers
    assert headers['Authorization'] == 'Basic ZHVtbXk6cGFzc3dvcmQ='


def test_no_header_without_setting():
    """Make sure that the endpoint has no authorization header if either username or password is missing"""
    client1 = PublicationDataApi("no-url", "dummy", None)
    client2 = PublicationDataApi("no-url", None, "abc")
    client3 = PublicationDataApi("no-url", None, None)
    headers = [ client.endpoint.base_headers for client in [client1, client2, client3] ]
    assert 'Authorization' not in headers[0]
    assert 'Authorization' not in headers[1]
    assert 'Authorization' not in headers[2]


def test_raise_missing_endpoint():
    """Remove env variable PDP_API and assert OSError"""
    with pytest.raises(ValueError):
        _ = PublicationDataApi(None, None, None)


def test_get_endpoint_from_env():
    """Test endpoint object creation"""
    client = PublicationDataApi("dummy", "dummy", "dummy")
    assert client.endpoint is not None

def test_operation_is_working():
    """Test that the query operation can be retrieved easily"""
    client = PublicationDataApi("dummy", "dummy", "dummy")
    op = client.query_op()
    assert op is not None
    assert isinstance(op, Operation)

def test_query_failure():
    """Mock server failure response and assert SystemExit"""
    with patch.object(PublicationDataApi, 'call_api', new=dummy_call_api):
        client = PublicationDataApi("dummy-url", "dummy-user", "dummy-password")
        with pytest.raises(PublicationDataApiException):
            _ = client.run_query("{query{}}")


def dummy_call_api(self, query, variables):
    return EXAMPLE_RESPONSE_ERROR



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['sgqlc', ]

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author="SRGSSR PDP Team",
    author_email='pdp@srgssr.ch',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Python client library to conveniently query the publication data of SRG/SSR",
    # entry_points={
    #     'console_scripts': [
    #         'srgssr_publication_data_api=srgssr_publication_data_api.cli:main',
    #     ],
    # },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history, # TODO rewrite this one
    include_package_data=True,
    keywords='srgssr metadata pdp publication data',
    name='srgssr-publication-data-api',
    packages=find_packages(include=['srgssr_publication_data_api']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/SRGSSR/pdp_graphql_client_python',
    version='0.4.1',
    zip_safe=False,
)

python_requires='>=3.6'
